@@ -1,7 +1,7 @@
 package CORBA::MICO::Misc;
 require Exporter;
 
-require Gtk;
+require Gtk2;
 
 use strict;
 
@@ -25,7 +25,7 @@ use vars qw($ctree_pixmaps);
 # Force updating of screen (process pending events)
 # Return value: TRUE if main_quit has been called, FALSE else
 sub process_pending {
-  my $ret = Gtk->main_iteration() while Gtk->events_pending();
+  my $ret = Gtk2->main_iteration() while Gtk2->events_pending();
   return $ret;
 }
 
@@ -36,8 +36,8 @@ sub process_pending {
 # Return value: TRUE if main_quit has been called, FALSE else
 #--------------------------------------------------------------------
 sub cursor_watch {
-  # return cursor_set(Gtk::Gdk::GDK_WATCH, @_);
-  return cursor_set(Gtk::Gdk::Cursor->new(150), @_);
+  # return cursor_set(Gtk2::Gdk::GDK_WATCH, @_);
+  return cursor_set(Gtk2::Gdk::Cursor->new('watch'), @_);
 }
 
 #--------------------------------------------------------------------
@@ -47,8 +47,8 @@ sub cursor_watch {
 # Return value: TRUE if main_quit has been called, FALSE else
 #--------------------------------------------------------------------
 sub cursor_hand2 {
-  # return cursor_set(Gtk::Gdk::GDK_HAND2, @_);
-  return cursor_set(Gtk::Gdk::Cursor->new(60), @_);
+  # return cursor_set(Gtk2::Gdk::GDK_HAND2, @_);
+  return cursor_set(Gtk2::Gdk::Cursor->new('hand2'), @_);
 }
 
 #--------------------------------------------------------------------
@@ -92,7 +92,7 @@ sub cursor_set {
 #--------------------------------------------------------------------
 sub select_file {
   my ($title, $def_name, $show_fileop, $callback, @udata) = @_;
-  my $dialog = new Gtk::FileSelection($title);
+  my $dialog = new Gtk2::FileSelection($title);
   $dialog->ok_button->signal_connect(
                           'clicked', 
                           sub { 
@@ -101,11 +101,11 @@ sub select_file {
                             }
                           });
   $dialog->cancel_button->signal_connect('clicked', sub { $dialog->destroy() });
-  $dialog->position('mouse');
+  $dialog->set_position('mouse');
   $dialog->set_filename($def_name) if $def_name;
   $dialog->hide_fileop_buttons()   unless $show_fileop;
   $dialog->show_all();
-  $dialog->grab_remove();
+  Gtk2->grab_remove($dialog);
 }
 
 #--------------------------------------------------------------------
@@ -113,39 +113,39 @@ sub select_file {
 #--------------------------------------------------------------------
 sub warning {
   my ($text) = @_;
-  my $dialog = new Gtk::Dialog;
-  $dialog->position('mouse');
-  my $label = new Gtk::Label($text);
+  my $dialog = new Gtk2::Dialog;
+  $dialog->set_position('mouse');
+  my $label = new Gtk2::Label($text);
   $label->set_padding(10, 10);
   $dialog->vbox()->pack_start($label, 1, 1, 0);
 
-  my $bbox = new Gtk::HButtonBox;
+  my $bbox = new Gtk2::HButtonBox;
   $bbox->set_spacing(5);
   $bbox->set_layout('end');
   $dialog->action_area()->pack_start($bbox, 1, 1, 0);
 
-  my $ok_button = new_with_label Gtk::Button("OK");
+  my $ok_button = new_with_label Gtk2::Button("OK");
   $ok_button->signal_connect('clicked', sub { $dialog->destroy() });
   $ok_button->can_default(1);
   $bbox->pack_end($ok_button, 0, 0, 0);
   $ok_button->grab_default();
      
   $dialog->grab_add();
-  $dialog->signal_connect('destroy', sub { $dialog->grab_remove() });
+  $dialog->signal_connect('destroy', sub { Gtk2->grab_remove($dialog) });
   $dialog->show_all();
 }
 
 #--------------------------------------------------------------------
-# Create status line, return corresponding Gtk::Label widget
+# Create status line, return corresponding Gtk2::Label widget
 #--------------------------------------------------------------------
 sub status_line_create {
   my $widget;
   if(0) {
-    $widget = new Gtk::Label('');
+    $widget = new Gtk2::Label('');
     $widget->set_justify('left');
   }
   else {
-    $widget = new Gtk::Entry();
+    $widget = new Gtk2::Entry();
     $widget->set_editable(0);
   }
   return $widget;
@@ -164,107 +164,43 @@ sub status_line_write {
 }
 
 #--------------------------------------------------------------------
-# pixmaps for CTree
-my @book_open_xpm = (
-"16 16 4 1",
-"       c None s None",
-".      c black",
-"X      c #808080",
-"o      c white",
-"                ",
-"  ..            ",
-" .Xo.    ...    ",
-" .Xoo. ..oo.    ",
-" .Xooo.Xooo...  ",
-" .Xooo.oooo.X.  ",
-" .Xooo.Xooo.X.  ",
-" .Xooo.oooo.X.  ",
-" .Xooo.Xooo.X.  ",
-" .Xooo.oooo.X.  ",
-"  .Xoo.Xoo..X.  ",
-"   .Xo.o..ooX.  ",
-"    .X..XXXXX.  ",
-"    ..X.......  ",
-"     ..         ",
-"                ");
-
-my @book_closed_xpm = (
-"16 16 6 1",
-"       c None s None",
-".      c black",
-"X      c red",
-"o      c yellow",
-"O      c #808080",
-"#      c white",
-"                ",
-"       ..       ",
-"     ..XX.      ",
-"   ..XXXXX.     ",
-" ..XXXXXXXX.    ",
-".ooXXXXXXXXX.   ",
-"..ooXXXXXXXXX.  ",
-".X.ooXXXXXXXXX. ",
-".XX.ooXXXXXX..  ",
-" .XX.ooXXX..#O  ",
-"  .XX.oo..##OO. ",
-"   .XX..##OO..  ",
-"    .X.#OO..    ",
-"     ..O..      ",
-"      ..        ",
-"                ");
-
-my @mini_page_xpm = (
-"16 16 4 1",
-"       c None s None",
-".      c black",
-"X      c white",
-"o      c #808080",
-"                ",
-"   .......      ",
-"   .XXXXX..     ",
-"   .XoooX.X.    ",
-"   .XXXXX....   ",
-"   .XooooXoo.o  ",
-"   .XXXXXXXX.o  ",
-"   .XooooooX.o  ",
-"   .XXXXXXXX.o  ",
-"   .XooooooX.o  ",
-"   .XXXXXXXX.o  ",
-"   .XooooooX.o  ",
-"   .XXXXXXXX.o  ",
-"   ..........o  ",
-"    oooooooooo  ",
-"                ");
-
+# search handler for TreeView
+# (look for substring at any positioin within the column)
 #--------------------------------------------------------------------
-# Return pixmaps for CTree
-# In:  $widget  - a widget
-# Out: a hash containing the following values:
-#       (LEAF,        pixmap for leafs)
-#       (LEAF_MASK,   pixmap mask for leafs)
-#       (CLOSED,      pixmap for closed state)
-#       (CLOSED_MASK, pixmap maks for closed state)
-#       (OPEN,        pixmap for open state)
-#       (OPEN_MASK,   pixmap maks for open state)
-#--------------------------------------------------------------------
-sub ctree_pixmaps
-{
-  my $widget = shift;
-  if( not defined($ctree_pixmaps) ) {
-    # Widget should be realized to have defined GdkWindow
-    $widget->realize();
-    my ($b_open, $b_open_mask) = create_from_xpm_d Gtk::Gdk::Pixmap(
-                                    $widget->window, undef, @book_open_xpm);
-    my ($b_closed, $b_closed_mask) = create_from_xpm_d Gtk::Gdk::Pixmap(
-                                    $widget->window, undef, @book_closed_xpm);
-    my ($mini_page, $mini_page_mask) = create_from_xpm_d Gtk::Gdk::Pixmap(
-                                    $widget->window, undef, @mini_page_xpm);
-    $ctree_pixmaps->{'OPEN'} = $b_open;
-    $ctree_pixmaps->{'OPEN_MASK'} = $b_open_mask;
-    $ctree_pixmaps->{'CLOSED'} = $b_closed;
-    $ctree_pixmaps->{'CLOSED_MASK'} = $b_closed_mask;
-    $ctree_pixmaps->{'LEAF'} = $mini_page;
-    $ctree_pixmaps->{'LEAF_MASK'} = $mini_page_mask;
+sub ctree_std_search {
+  my ($model, $column, $key, $iter, $ud) = @_;
+  my ($text) = $model->get($iter, $column);
+  my $rv = 1;
+  # use eval so as illegal regexp may an cause exception
+  if( $ud->{REGEXP} ) {
+    eval { $rv = $text !~ /$key/i };
   }
-  return $ctree_pixmaps;
+  else {
+    eval { $rv = $text !~ /\Q$key\E/i };
+  }
+  return $rv;
 }
+
+#--------------------------------------------------------------------
+# Emit signal start_interactive_search when CTRL_F is pressed
+#--------------------------------------------------------------------
+sub ctree_kpress {
+  my ($w, $event, $ud) = @_;
+  my $key = Gtk2::Gdk->keyval_name($event->keyval());
+  if( $key eq 'f' ) {
+    $ud->{REGEXP} = 0;
+  }
+  elsif( $key eq 'r' ) {
+    $ud->{REGEXP} = 1;
+  }
+  else {
+    return 0;
+  }
+  my $st = $event->state();
+  if( $#$st == 0 && $st->[0] eq 'control-mask' ) {
+    $w->signal_emit('start_interactive_search');
+    return 1;
+  }
+  return 0;
+}
+
